@@ -2,6 +2,23 @@ MAPPERS_DEFINED = [False]
 
 
 
+BUMP_INTERVALS = [
+    ("hour", "hour"),
+    ("day", "day"), 
+    ("week", "week"),
+    ("2week", "2 weeks"),
+    ("month", "month"),
+    ("year", "year")
+    ]
+
+
+
+
+def is_valid_tag(tag):
+    return tag.isalnum()
+
+
+
 
 class DataModel(object):
     def __init__(self):
@@ -12,7 +29,8 @@ class DataModel(object):
 
         from sqlalchemy import Table, Column, \
                 Integer, Float, Text, UnicodeText, Unicode, ForeignKey, \
-                MetaData
+                MetaData, Boolean
+        from sqlalchemy.types import Enum
 
         cls = DataModel
 
@@ -31,7 +49,18 @@ class DataModel(object):
                 Column('id', Integer, primary_key=True),
                 Column('item_id', Integer, ForeignKey('items.id')),
                 Column('timestamp', Float, index=True),
-                Column('contents', UnicodeText())
+                Column('contents', UnicodeText()),
+
+                # all-day events are saved with start/end dates at 0:00 
+                # in the GMT timezone
+                Column('all_day', Boolean()),
+
+                Column('start_date', Float()),
+                Column('end_date', Float()),
+                Column('bump_interval', 
+                    Enum(*[key for key, val in BUMP_INTERVALS])),
+                Column('hide_until', Float()),
+                Column('highlight_at', Float()),
                 )
 
         cls.itemversions_tags = Table('itemversions_tags', self.metadata,
@@ -41,7 +70,7 @@ class DataModel(object):
 
         cls.vieworderings = Table('vieworderings', self.metadata,
                 Column('id', Integer, primary_key=True),
-                Column('tagset', Text()), # misnomer, by now: normalized query string
+                Column('norm_query', Text()),
                 Column('timestamp', Float, index=True),
                 )
 
@@ -98,11 +127,22 @@ class Item(object):
 
 _html_cache = {}
 class ItemVersion(object):
-    def __init__(self, item, timestamp, tags, contents):
-        self.item = item
-        self.timestamp = timestamp
-        self.tags = tags
-        self.contents = contents
+    def __init__(self, **kwargs):
+        self.item = kwargs.pop("item")
+        self.timestamp = kwargs.pop("timestamp")
+        self.tags = kwargs.pop("tags")
+        self.contents = kwargs.pop("contents")
+
+        self.all_day = kwargs.pop("all_day", False)
+        self.start_date = kwargs.pop("start_date", None)
+        self.end_date = kwargs.pop("end_date", None)
+        self.hide_until = kwargs.pop("hide_until", None)
+        self.highlight_at = kwargs.pop("highlight_at", None)
+        self.bump_interval = kwargs.pop("bump_interval", None)
+
+        if kwargs:
+            raise TypeError(
+                    "invalid keyword args: '%s'" % ",".join(kwargs.iterkeys()))
 
     def as_json(self):
         return {"id": self.item.id,
@@ -110,6 +150,13 @@ class ItemVersion(object):
                 "timestamp": self.timestamp,
                 "tags": [tag.name for tag in self.tags],
                 "contents": self.contents,
+
+                "all_day": self.all_day,
+                "start_date": self.start_date,
+                "end_date": self.end_date,
+                "hide_until": self.hide_until,
+                "highlight_at": self.highlight_at,
+                "bump_interval": self.bump_interval,
                 }
 
     def copy(self, item=None, timestamp=None, tags=None, contents=None):
@@ -145,8 +192,8 @@ class ItemVersion(object):
 
 
 class ViewOrdering(object):
-    def __init__(self, tagset, timestamp):
-        self.tagset = tagset
+    def __init__(self, norm_query, timestamp):
+        self.norm_query = norm_query
         self.timestamp = timestamp
 
 
@@ -190,10 +237,15 @@ def find_tags(session, tags, create_them):
 
 
 
-def store_itemversion(dbsession, contents, tags, item_id=None, timestamp=None):
+def store_itemversion(dbsession, **kwargs):
     import re
     from htmlentitydefs import name2codepoint
 
+    contents = kwargs.get("contents")
+    item_id = kwargs.get("id")
+    timestamp = kwargs.get("timestamp")
+    tags = kwargs.get("tags")
+
     if contents is not None:
         def replace_special_char(match):
             try:
@@ -214,12 +266,15 @@ def store_itemversion(dbsession, contents, tags, item_id=None, timestamp=None):
         from time import time
         timestamp = time()
 
-    itemversion = ItemVersion(
-            item,
-            timestamp,
-            find_tags(dbsession, tags, create_them=True),
-            contents,
-            )
+    props = kwargs.copy()
+    if "id" in props:
+        del props["id"]
+    props.update(
+            item=item,
+            timestamp=timestamp,
+            tags=find_tags(dbsession, tags, create_them=True),
+            contents=contents)
+    itemversion = ItemVersion(**props)
     dbsession.add(itemversion)
 
     return itemversion
@@ -231,6 +286,11 @@ def store_itemversion(dbsession, contents, tags, item_id=None, timestamp=None):
 class SQLifyQueryVisitor(object):
     def __init__(self, session):
         self.session = session
+        self.use_hide_until = True
+        self.sort_by_date = False
+
+    def visit_id_query(self, q):
+        return ItemVersion.item_id == q.id
 
     def visit_tag_query(self, q):
         tags = self.session.query(Tag).filter_by(name=q.name)
@@ -246,6 +306,7 @@ class SQLifyQueryVisitor(object):
 
         cnt = tags.count()
         if cnt == 0:
+            from sqlalchemy.sql import literal
             return literal(True)
         elif tags.count() == 1:
             return ItemVersion.tags.any(id=tags[0].id)
@@ -275,6 +336,16 @@ class SQLifyQueryVisitor(object):
         else:
             return ItemVersion.timestamp > q.timestamp
 
+    def visit_dated_query(self, q):
+        from sqlalchemy.sql import or_
+        return or_(ItemVersion.start_date != None,
+                ItemVersion.end_date != None)
+
+    def visit_no_hide_query(self, q):
+        self.use_hide_until = False
+
+    def visit_sort_by_date(self, q):
+        self.sort_by_date = True
 
 
 
@@ -316,7 +387,7 @@ def query_itemversions(session, model, parsed_query, max_timestamp=None):
 
     # find view ordering
     view_orderings = (session.query(ViewOrdering)
-            .filter_by(tagset=str(parsed_query))
+            .filter_by(norm_query=unicode(parsed_query))
             .order_by(ViewOrdering.timestamp.desc())
             .limit(1)).all()
 
@@ -341,14 +412,26 @@ def query_itemversions(session, model, parsed_query, max_timestamp=None):
                 model.itemversions.c.item_id==vo_entries.c.item_id)
 
     from synoptic.datamodel import SQLifyQueryVisitor
-    where = and_(
+    visitor = SQLifyQueryVisitor(session)
+
+    conditions = [
             model.itemversions.c.contents != None,
-            parsed_query.visit(SQLifyQueryVisitor(session))
-            )
+            parsed_query.visit(visitor)
+            ]
+
+    if visitor.use_hide_until:
+        from time import time
+        conditions.append(
+                or_(ItemVersion.hide_until < time(),
+                    ItemVersion.hide_until == None))
+
+    where = and_(*conditions)
 
     result = select([model.itemversions], from_obj=[from_obj]).where(where)
 
-    if have_view_ordering:
+    if visitor.sort_by_date:
+        result = result.order_by(ItemVersion.start_date)
+    elif have_view_ordering:
         # add the ordering clause
         result = result.order_by(vo_entries.c.weight)
     else:
@@ -371,7 +454,7 @@ class ViewOrderingHandler:
 
     def has_ordering(self):
         view_orderings = (self.session.query(ViewOrdering)
-                .filter_by(tagset=str(self.parsed_query))
+                .filter_by(norm_query=str(self.parsed_query))
                 .order_by(ViewOrdering.timestamp.desc())
                 .limit(1)).all()
 
