@@ -1,3 +1,10 @@
+var UP_TO_DATE_TIMEOUT = 60*1000;
+var DAY_NAMES = ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday",
+    "Saturday"];
+var SHORT_DAY_NAMES = ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"];
+var SHORT_MONTH_NAMES = ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul",
+    "Aug", "Sep", "Oct", "Nov", "Dec"];
+
 var msgnum = 0;
 
 function busy(what)
@@ -16,10 +23,9 @@ function set_message(what, timeout)
   var my_msg = $("#msg_"+msgnum);
 
   if (timeout  == undefined)
-    timeout = "10s";
+    timeout = 10*1000;
 
-  $("#messagearea").oneTime(timeout, function()
-      { my_msg.remove(); })
+  window.setTimeout(function() { my_msg.remove(); }, 10*1000)
 }
 
 function report_error(what)
@@ -30,7 +36,7 @@ function report_error(what)
 
 
 
-function zero_pad(s, min_length) 
+function zero_pad(s, min_length)
 {
   while (s.length < min_length)
     s = "0"+s;
@@ -40,11 +46,70 @@ function zero_pad(s, min_length)
 
 
 
-// String extension  -----------------------------------------------------------
+function format_editable_timestamp(timestamp, all_day)
+{
+  var dt = new Date(timestamp*1000);
+
+  if (all_day)
+  {
+    var result = sprintf("%d/%d/%d",
+      dt.getUTCMonth()+1, dt.getUTCDate(), dt.getUTCFullYear());
+  }
+  else
+  {
+    var result = sprintf("%d/%d/%d",
+      dt.getMonth()+1, dt.getDate(), dt.getFullYear());
+
+    if (!(dt.getHours() == 0 && dt.getMinutes() == 0 && dt.getSeconds() == 0))
+      result += sprintf(" %d:%02d", dt.getHours(), dt.getMinutes());
+  }
+
+  return result;
+}
+
+
+
+
+function format_timestamp(timestamp, all_day)
+{
+  var dt = new Date(timestamp*1000);
+  var now = new Date();
+
+  if (all_day)
+  {
+    var result = SHORT_DAY_NAMES[dt.getUTCDay()];
+
+    if (dt.getUTCFullYear() == now.getUTCFullYear())
+      result += sprintf(" %s %d", SHORT_MONTH_NAMES[dt.getUTCMonth()], dt.getUTCDate());
+    else
+      result += sprintf(" %s %d %d",
+        SHORT_MONTH_NAMES[dt.getUTCMonth()], dt.getUTCDate(), dt.getUTCFullYear());
+  }
+  else
+  {
+    var result = SHORT_DAY_NAMES[dt.getDay()];
+
+    if (dt.getFullYear() == now.getFullYear())
+      result += sprintf(" %s %d", SHORT_MONTH_NAMES[dt.getMonth()], dt.getDate());
+    else
+      result += sprintf(" %s %d %d",
+        SHORT_MONTH_NAMES[dt.getMonth()], dt.getDate(), dt.getFullYear());
+
+    if (!(dt.getHours() == 0 && dt.getMinutes() == 0 && dt.getSeconds() == 0))
+      result += sprintf(" %d:%02d", dt.getHours(), dt.getMinutes());
+  }
+
+  return result;
+}
+
+
+
+
+// {{{ String extension
 String.method("allreplace", function(from, to)
 {
   var last, current = this;
-  do 
+  do
   {
     last = current;
     current = last.replace(from, to);
@@ -66,10 +131,9 @@ function str(val)
     return val.toString();
 }
 
+// }}}
 
-
-
-// misc tools -----------------------------------------------------------------
+// {{{ misc tools
 function re_escape(text) {
   if (!arguments.callee.sRE) {
     var specials = [
@@ -94,10 +158,9 @@ function find_in_array(ary, item)
   return -1;
 }
 
+// }}}
 
-
-
-// ItemManager  --------------------------------------------------------------------
+// {{{ ItemManager
 function ItemManager(mgr, arg, is_historic)
 {
   this.manager = mgr;
@@ -111,6 +174,8 @@ function ItemManager(mgr, arg, is_historic)
     this.id = null;
   else
     this.set_from_obj(arg, is_historic);
+
+  this.timeout_id = null;
 }
 
 
@@ -125,11 +190,33 @@ ItemManager.method("set_from_obj", function(arg, is_historic)
   this.contents = arg.contents;
   this.contents_html = arg.contents_html;
 
+  this.all_day = arg.all_day;
+  this.start_date = arg.start_date;
+  this.end_date = arg.end_date;
+  this.bump_interval = arg.bump_interval;
+  this.hide_until = arg.hide_until;
+  this.highlight_at = arg.highlight_at;
+
   this.is_historic = is_historic;
 });
 
 
 
+ItemManager.method("as_json", function()
+{
+  return {
+    id: this.id,
+    tags: this.tags,
+    contents: this.contents,
+
+    all_day: this.all_day,
+    start_date: this.start_date,
+    end_date: this.end_date,
+    bump_interval: this.bump_interval,
+    hide_until: this.hide_until,
+    highlight_at: this.highlight_at
+  }
+});
 
 ItemManager.method("call_with_item_div", function(inserter, when_created)
 {
@@ -162,9 +249,9 @@ ItemManager.method("fill_item_div", function(history)
       );
     $('#new_'+self.id).click(function(){ self.begin_edit() });
   }
-  else 
+  else
   {
-    // generate history html, if requested
+    // {{{ generate history html, if requested / available
     if (history != null)
     {
       var hist_html = "";
@@ -191,11 +278,34 @@ ItemManager.method("fill_item_div", function(history)
       }
       hist_html = '<ul>'+hist_html+"</ul>";
     }
+    // }}}
 
     var contents_html = self.contents_html;
     if (contents_html == null)
       contents_html = "(deleted)";
 
+    function get_date_info()
+    {
+      var date_info = "";
+      var now = new Date();
+
+      if (self.start_date)
+        date_info += ('<span class="start-date">'
+            +format_timestamp(self.start_date, self.all_day)+'</span>');
+      if (self.end_date)
+        date_info += ('<span class="end-date"> &mdash;'
+            +format_timestamp(self.end_date, self.all_day)+'</span>');
+      if (self.hide_until && now.getTime()/1000 < self.hide_until)
+      {
+        date_info += ('<span class="hide-date">Hidden until '
+            +format_timestamp(self.hide_until)+'</span>');
+      }
+
+      if (date_info.length)
+        return '<div class="date-info">'+date_info+'</div>';
+      else
+        return '';
+    }
 
     // generate item html
     if (!self.is_historic)
@@ -214,10 +324,12 @@ ItemManager.method("fill_item_div", function(history)
         '</div>'+
         '<div id="item_history_[id]" class="item_history ui-corner-all">'+hist_html+'</div>'+
         '</div>'+
+        '[dateinfo]'+
         '<div id="item_contents_[id]" class="itemcontents">[contents]</div>'
         ).allreplace('[id]', self.id)
         .allreplace('[tags]', format_tag_links(self.tags))
         .allreplace('[contents]', contents_html)
+        .allreplace('[dateinfo]', get_date_info())
         );
       $('#edit_'+self.id).click(function(){ self.begin_edit() });
       $('#delete_'+self.id).click(function(){ self.do_delete() });
@@ -237,45 +349,43 @@ ItemManager.method("fill_item_div", function(history)
         '</div>'+
         '<div id="item_history_[id]" class="item_history ui-corner-all">'+hist_html+'</div>'+
         '</div>'+
+        '[dateinfo]'+
         '<div id="item_contents_[id]" class="itemcontents">[contents]</div>'
         ).allreplace('[id]', self.id)
         .allreplace('[tags]', format_tag_links(self.tags))
         .allreplace('[contents]', contents_html)
+        .allreplace('[dateinfo]', get_date_info())
         );
       $('#btn_revert_'+self.id).click(function()
-        { 
+        {
+          var data = self.as_json();
+          data.current_query = $("#search").val();
+
           $.ajax({
             type: 'POST',
             dataType: 'json',
             url: 'item/store',
-            data: {json: JSON.stringify({
-              id: self.id,
-              tags: self.tags,
-              contents: self.contents,
-              current_query: $("#search").val()
-            })},
+            data: {json: JSON.stringify(data)},
             error: function(req, stat, err) { report_error("Revert failed."); },
-            success: function(data, msg) { 
-              set_message("Revert successful."); 
+            success: function(data, msg) {
+              set_message("Revert successful.");
               update_tag_clouds();
             }
           });
         });
       $('#btn_copy_to_present_'+self.id).click(function()
-        { 
+        {
+          var data = self.as_json();
+          data.current_query = $("#search").val();
+
           $.ajax({
             type: 'POST',
             dataType: 'json',
             url: 'item/store',
-            data: {json: JSON.stringify({
-              id: null,
-              tags: self.tags,
-              contents: self.contents,
-              current_query: $("#search").val()
-            })},
+            data: {json: JSON.stringify(data)},
             error: function(req, stat, err) { report_error("Copy failed."); },
-            success: function(data, msg) { 
-              set_message("Copy successful."); 
+            success: function(data, msg) {
+              set_message("Copy successful.");
               update_tag_clouds();
             }
           });
@@ -310,7 +420,7 @@ ItemManager.method("fill_item_div", function(history)
                   self.set_from_obj(data, is_historic);
                   self.fill_item_div(history);
                 },
-                error: function(req, stat, err) 
+                error: function(req, stat, err)
                 { report_error("Error retrieving version."); },
             });
           });
@@ -319,17 +429,55 @@ ItemManager.method("fill_item_div", function(history)
       for (var i = 0; i < history.length; ++i)
         bind_version_click_handler(history[i], i != 0);
     }
+
+    if (self.highlight_at)
+    {
+      var now = new Date().getTime()/1000;
+
+      if (now > self.highlight_at)
+      {
+        self.div.addClass("highlighted");
+      }
+      else
+      {
+        // set up a timer to highlight the element when it's time
+        self.div.removeClass("highlighted");
+
+        if (self.timeout_id != null)
+          window.clearTimeout(self.timeout_id);
+
+        var timeout_value = (self.highlight_at - now)*1000;
+        if (timeout_value < 7*24*3600*1000) /* at most a week */
+        {
+          self.timeout_id = window.setTimeout(function () {
+            self.div.addClass("highlighted");
+          }, timeout_value);
+
+          // make sure the timeout gets killed when the page is refilled
+          self.manager.item_timeouts.push(self.timeout_id);
+        }
+      }
+    }
   }
 });
 
 
+ItemManager.method("set_highlight_for_timestamp", function(timestamp)
+{
+  if (timestamp > self.highlight_at)
+    self.div.addClass("highlighted");
+  else
+    self.div.removeClass("highlighted");
+});
+
+
 function item_toggle_collapsed(div)
 {
   $(div).toggleClass("collapsed");
 }
 
 
-ItemManager.method("toggle_history", function() 
+ItemManager.method("toggle_history", function()
 {
   var self = this;
 
@@ -339,9 +487,9 @@ ItemManager.method("toggle_history", function()
     // make sure we're showing the current version
     self.load_from_server(
       function() { self.fill_item_div(); }
-      ); 
+      );
   }
-  else 
+  else
   {
     var histdiv = $(self.div).find(".item_history");
 
@@ -365,12 +513,20 @@ ItemManager.method("toggle_history", function()
 
 
 var edit_count = 0;
-ItemManager.method("begin_edit", function() 
+ItemManager.method("begin_edit", function()
 {
   ++edit_count;
   var self = this;
 
-  var text_height = self.div.find(".itemcontents").height();
+  var text_height;
+  if (self.div.hasClass("editing"))
+  {
+    // a repeat edit, keep previous height
+    text_height = self.div.find("textarea").height();
+  }
+  else
+    text_height = self.div.find(".itemcontents").height();
+
   if (text_height < 150)
     text_height = 150;
 
@@ -383,12 +539,38 @@ ItemManager.method("begin_edit", function()
     '</div>'+
     '<div class="itemcontents">'+
     '<textarea id="editor_[id]" cols="80"></textarea>'+
+    '</div>'+
+    '<div class="datecontrols">'+
+    '<div class="whencontrols">'+
+    '<label for="edit_date_[id]">Date:</label>'+
+    '<input type="text" id="edit_date_[id]" size="15"/>'+
+    '<label for="edit_end_[id]">End:</label>'+
+    '<input type="text" id="edit_end_[id]" size="15"/>'+
+    '<input type="checkbox" id="chk_all_day_[id]"/>'+
+    '<label for="chk_all_day_[id]">All day</label>'+
+    '</div>'+
+    '<div class="remindcontrols">'+
+    '<label for="edit_hide_until_[id]">Hide Until:</label>'+
+    '<input type="text" id="edit_hide_until_[id]" size="15"/>'+
+    '<label for="edit_highlight_at_[id]">Highlight at:</label>'+
+    '<input type="text" id="edit_highlight_at_[id]" size="15"/>'+
+    '</div>'+
+    '<div class="bumpcontrols">'+
+    '<label for="sel_bump_interval_[id]">Bump interval:</label>'+
+    '<select id="sel_bump_interval_[id]" />'+
+    '<input type="checkbox" id="chk_bump_and_copy_[id]" />'+
+    '<label for="chk_bump_and_copy_[id]">Save, then bump new</label> '+
+    '<input type="button" id="btn_bump_bwd_[id]" value="&lt; Bump"/>'+
+    '<input type="button" id="btn_bump_fwd_[id]" value="Bump &gt;"accesskey="b"/>'+
+    '</div>'+
     '</div>'
     ).allreplace("[id]",  self.id)
     );
   self.div.addClass("editing");
 
-  // default to current search tags
+  // {{{ initial content setup
+
+  // default to current search tags if new item
   if (self.id == null)
   {
     var default_tags = self.manager.query_tags.join(",");
@@ -404,61 +586,156 @@ ItemManager.method("begin_edit", function()
 
   $("#editor_"+self.id).val(self.contents);
 
-  self.manager.install_focus_handlers($("#editor_"+self.id));
   $("#editor_"+self.id).get(0).focus();
   $("#editor_"+self.id).height(text_height);
 
-  self.manager.install_focus_handlers($("#edit_tags_"+self.id));
-  $("#edit_tags_"+self.id).autocomplete("tags/get_filter",
-      { 
-        delay: 100, 
-        multiple:true, 
-        // autoFill: true, 
-        cacheLength:1,
-        multipleSeparator:","
-      });
+  function split( val ) { return val.split( /,\s*/ ); }
+  function extractLast( term ) { return split(term).pop(); }
+
+  $("#edit_tags_"+self.id)
+    // don't navigate away from the field on tab when selecting an item
+    .bind("keydown", function(event) {
+      if ( event.keyCode === $.ui.keyCode.TAB &&
+          $( this ).data( "autocomplete" ).menu.active ) {
+        event.preventDefault();
+      }
+    })
+    .autocomplete({
+      minLength: 1,
+      delay: 100,
+      source: function(request, response) {
+        $.getJSON("tags/get_filter", { q: extractLast(request.term) }, response);
+      },
+      focus: function() {
+        // prevent value inserted on focus
+        return false;
+      },
+      select: function(event, ui) {
+        var terms = split(this.value);
+        terms.pop();
+        terms.push( ui.item.value );
+        terms.push( "" );
+        this.value = terms.join( "," );
+        return false;
+      }
+    });
+
+  // {{{ date - related
+
+  $("#edit_date_"+self.id).datepicker({ constrainInput: false, showOn: 'button' });
+  $("#edit_end_"+self.id).datepicker({ constrainInput: false, showOn: 'button' });
+  $("#edit_hide_until_"+self.id).datepicker({ constrainInput: false, showOn: 'button' });
+  $("#edit_highlight_at_"+self.id).datepicker({ constrainInput: false, showOn: 'button' });
+  $(".ui-datepicker-trigger", self.div).attr("tabindex", -1);
+
+  function set_date_val(name, value, all_day_affected)
+  {
+    if (value) $(name).val(format_editable_timestamp(
+          value, all_day_affected && self.all_day));
+  }
+
+  $("#chk_all_day_"+self.id).get(0).checked = self.all_day;
+  set_date_val("#edit_date_"+self.id, self.start_date, true);
+  set_date_val("#edit_end_"+self.id, self.end_date, true);
+  set_date_val("#edit_hide_until_"+self.id, self.hide_until, false);
+  set_date_val("#edit_highlight_at_"+self.id, self.highlight_at, false);
+
+  var bump_intv_html;
+
+  for (var i in BUMP_INTERVALS)
+  {
+    bump_intv_html +=
+      '<option value="'+BUMP_INTERVALS[i][0]+'">'+BUMP_INTERVALS[i][1]+"</option>";
+  }
+
+  $("#sel_bump_interval_"+self.id).html(bump_intv_html);
+  if (self.bump_interval)
+    $("#sel_bump_interval_"+self.id).val(self.bump_interval);
+
+  // }}}
+
+  // }}}
+
   self.div.get(0).scrollIntoView(false);
 
-  $("#edit_ok_"+self.id).click(function(){
-    self.div.removeClass("editing");
+  // {{{ button responses
 
+  // {{{ basic save functionality
+
+  function save_item(options)
+  {
     var tags = parse_tags($("#edit_tags_"+self.id).val());
 
-    for (var i = 0; i<tags.length; ++i)
-      if (!is_valid_tag(tags[i]))
-      {
-        alert("Invalid tag: '[tag]'".allreplace("[tag]", tags[i]));
-        return;
-      }
-
-    self.contents = $("#editor_"+self.id).val();
+    var new_id = self.id;
+    if (typeof(new_id) != 'number')
+      new_id = null;
 
     $.ajax({
       type: 'POST',
       dataType: 'json',
       url: 'item/store',
       data: {json: JSON.stringify({
-        id: self.id,
+        id: new_id,
         tags: tags,
-        contents: self.contents,
-        current_query: $("#search").val()
+
+        contents: $("#editor_"+self.id).val(),
+        current_query: $("#search").val(),
+
+        all_day: $("#chk_all_day_"+self.id).get(0).checked,
+        start_date: $("#edit_date_"+self.id).val(),
+        end_date: $("#edit_end_"+self.id).val(),
+        hide_until: $("#edit_hide_until_"+self.id).val(),
+        highlight_at: $("#edit_highlight_at_"+self.id).val(),
+        bump_interval: $("#sel_bump_interval_"+self.id).val()
       })},
 
-      error: function(req, stat, err) 
+      success: options["success"],
+      error: options["error"],
+    });
+  }
+
+  // }}}
+
+  $("#edit_ok_"+self.id).click(function(){
+    $("input", self.div).attr("disabled", true);
+
+    self.manager.check_if_results_are_current();
+    ++self.manager.up_to_date_check_inhibitions;
+
+    var new_contents = $("#editor_"+self.id).val();
+
+    save_item({
+      error: function(req, stat, err)
       {
+        self.div.removeClass("editing");
+
         --edit_count;
-        set_message("Saving failed.");
+        alert("Saving failed.\n"+req.responseText);
+        --self.manager.up_to_date_check_inhibitions;
         self.begin_edit();
+
+        // preserve contents in case saving fails
+        $("#editor_"+self.id).val(new_contents);
       },
 
-      success: function(data, msg) 
+      success: function(data, msg)
       {
+        self.div.removeClass("editing");
+
         --edit_count;
         var prev_id = self.id;
         self.set_from_obj(data);
         self.id = data.id;
         self.call_with_item_div(function(html){ self.div.replaceWith(html); });
         self.manager.set_cursor_to(self.div);
+
+        // Don't trigger change notification.
+        //
+        // There's a race condition here. Someone else might be changing items
+        // while we are doing the same. Not very likely, though--ok for our
+        // purposes.
+        self.manager.update_result_hash_and_uninhibit_check();
+
         update_tag_clouds();
         if (prev_id == null)
         {
@@ -471,10 +748,85 @@ ItemManager.method("begin_edit", function()
   $("#edit_cancel_"+self.id).click(function(){
     --edit_count;
     self.div.removeClass("editing");
-
     self.fill_item_div();
     self.manager.redraw_cursor();
   });
+
+  function bump(dir)
+  {
+    $.ajax({
+      type: 'POST',
+      dataType: 'json',
+      url: 'item/datebump',
+      data: {json: JSON.stringify({
+        id: self.id,
+        bump_direction: dir,
+        start_date: $("#edit_date_"+self.id).val(),
+        end_date: $("#edit_end_"+self.id).val(),
+        hide_until: $("#edit_hide_until_"+self.id).val(),
+        highlight_at: $("#edit_highlight_at_"+self.id).val(),
+        bump_interval: $("#sel_bump_interval_"+self.id).val()
+      })},
+
+      error: function(req, stat, err)
+      {
+        set_message("Bump server request failed.");
+      },
+
+      success: function(data, msg)
+      {
+        set_date_val("#edit_date_"+self.id, data.start_date);
+        set_date_val("#edit_end_"+self.id, data.end_date);
+        set_date_val("#edit_hide_until_"+self.id, data.hide_until);
+        set_date_val("#edit_highlight_at_"+self.id, data.highlight_at);
+      }
+    });
+  }
+
+  function bump_button_action(dir)
+  {
+    if (!$("#chk_bump_and_copy_"+self.id).get(0).checked)
+      bump(dir)
+    else
+    {
+      $("input", self.div).attr("disabled", true);
+      // save current, then bump, then edit again
+      var new_contents = $("#editor_"+self.id).val();
+
+      save_item({
+        error: function(req, stat, err)
+        {
+          set_message("Saving failed.");
+          self.begin_edit();
+
+          // preserve contents in case saving fails
+          $("#editor_"+self.id).val(new_contents);
+        },
+
+        success: function(data, msg)
+        {
+          self.set_from_obj(data);
+          self.id = "post_bump_"+self.id;
+          if (self.tags.indexOf("repeat") == "-1")
+            self.tags.push("repeat");
+          self.begin_edit();
+
+          bump(dir);
+
+          $("#chk_bump_and_copy_"+self.id).get(0).checked = true;
+          if (dir > 0)
+            $("#btn_bump_fwd_"+self.id).focus();
+          else
+            $("#btn_bump_bwd_"+self.id).focus();
+        }
+      });
+    }
+  }
+
+  $("#btn_bump_fwd_"+self.id).click(function () { bump_button_action(1); } );
+  $("#btn_bump_bwd_"+self.id).click(function () { bump_button_action(-1); } );
+
+  // }}}
 });
 
 
@@ -486,6 +838,8 @@ ItemManager.method("do_delete", function()
 
   self.manager.set_cursor_to(self.div.next());
   self.div.remove();
+  self.manager.check_if_results_are_current();
+  ++self.manager.up_to_date_check_inhibitions;
 
   $.ajax({
     type: 'POST',
@@ -496,18 +850,27 @@ ItemManager.method("do_delete", function()
       tags: [],
       contents: null
     })},
-    error: function(req, stat, err) { report_error("Delete failed."); },
-    success: function(data, msg) { 
+    error: function(req, stat, err) {
+             report_error("Delete failed.");
+             --self.manager.up_to_date_check_inhibitions;
+           },
+    success: function(data, msg) {
+      // Don't trigger change notification.
+      //
+      // There's a race condition here. Someone else might be changing items
+      // while we are doing the same. Not very likely, though--ok for our
+      // purposes.
+      self.manager.update_result_hash_and_uninhibit_check();
       update_tag_clouds();
     }
   });
 });
 
-ItemManager.method("load_from_server", function(when_done) 
+ItemManager.method("load_from_server", function(when_done)
 {
   var self = this;
   $.ajax({
-    url: "item/get_by_id", 
+    url: "item/get_by_id",
     data: {id: self.id},
     dataType:"json",
     success: function(obj, status)
@@ -523,10 +886,9 @@ ItemManager.method("load_from_server", function(when_done)
     });
 });
 
+// }}}
 
-
-
-// ItemCollectionManager ----------------------------------------------------------------
+// {{{ ItemCollectionManager
 function ItemCollectionManager()
 {
   var self = this;
@@ -540,6 +902,18 @@ function ItemCollectionManager()
   self.setup_toolbar();
 
   self.query_tags = [];
+
+  self.item_timeouts = [];
+
+  // catch out-of-date results
+  window.setTimeout(function() { self.check_if_results_are_current() },
+      UP_TO_DATE_TIMEOUT);
+  $("#ood_reload_btn").click(function() {
+    $("#out_of_date_notifier").addClass("ood_hidden");
+    self.reload();
+  })
+  self.up_to_date_check_inhibitions = 0;
+
 }
 
 
@@ -568,8 +942,8 @@ ItemCollectionManager.method("setup_history_handling", function()
 
   self.tsrange = null;
   $.getJSON("timestamp/get_range", function (json)
-    { 
-      self.tsrange = json; 
+    {
+      self.tsrange = json;
       self.set_history_slider(self.view_time);
     });
 
@@ -587,7 +961,7 @@ ItemCollectionManager.method("setup_history_handling", function()
           if (ui.value == 100)
             self.show_time(null);
           else
-            var new_time = self.tsrange.min 
+            var new_time = self.tsrange.min
               + ui.value/100.*(self.tsrange.max-self.tsrange.min);
             self.show_time(new_time);
         },
@@ -600,7 +974,7 @@ ItemCollectionManager.method("setup_history_handling", function()
             self.set_time(null, "slider");
           else
           {
-            var new_time = self.tsrange.min 
+            var new_time = self.tsrange.min
               + ui.value/100.*(self.tsrange.max-self.tsrange.min);
             self.set_time(new_time, "slider");
           }
@@ -625,8 +999,6 @@ ItemCollectionManager.method("setup_history_handling", function()
 
   $("#edit_date").val('');
   $("#history_time").html('present');
-
-  self.install_focus_handlers($("#edit_date"));
 });
 
 
@@ -634,10 +1006,13 @@ ItemCollectionManager.method("setup_history_handling", function()
 
 ItemCollectionManager.method("get_current_fragment", function()
 {
-  return escape(JSON.stringify({
+  // Need to double-escape because something is preventing unicode characters
+  // in URLs.
+
+  return escape(escape(JSON.stringify({
       query:$("#search").val(),
       timestamp:this.view_time
-    }));
+    })));
 });
 
 
@@ -647,7 +1022,8 @@ var nav_warning_inhibitor = 0;
 ItemCollectionManager.method("add_history_item", function()
 {
   ++nav_warning_inhibitor;
-  dhtmlHistory.add(this.get_current_fragment());
+  var frag = this.get_current_fragment();
+  dhtmlHistory.add(frag);
   --nav_warning_inhibitor;
 });
 
@@ -658,7 +1034,7 @@ ItemCollectionManager.method("handle_history_event", function(new_loc, dummy)
 {
   if (new_loc != "")
   {
-    new_loc = unescape(new_loc);
+    new_loc = unescape(unescape(new_loc));
     if (new_loc[0] == "{")
     {
       var loc_descriptor = JSON.parse(new_loc);
@@ -693,8 +1069,8 @@ ItemCollectionManager.method("setup_search", function()
       var is_updated = true;
 
       self.div.everyTime("100ms", "search_changewatch",
-        function() 
-        { 
+        function()
+        {
           var search = $("#search").val();
           if (search != last_search)
           {
@@ -710,7 +1086,7 @@ ItemCollectionManager.method("setup_search", function()
 
             if (unchanged_count > 3)
             {
-              self.update(); 
+              self.update();
               is_updated = true;
             }
           }
@@ -721,7 +1097,7 @@ ItemCollectionManager.method("setup_search", function()
     {
       self.div.stopTime("search_changewatch");
       self.add_history_item();
-      self.update(); 
+      self.update();
       --self.search_focused;
     });
   $("#btn_search_clear").click(function()
@@ -729,8 +1105,6 @@ ItemCollectionManager.method("setup_search", function()
       set_search("");
       $("#search").get(0).focus();
     });
-
-  self.install_focus_handlers($("#search"));
 });
 
 
@@ -740,9 +1114,8 @@ ItemCollectionManager.method("setup_keyboard_nav", function()
 {
   var self = this;
   self.cursor_at = null;
-  self.kb_shortcut_inhibitions = 0;
   self.search_focused = 0;
-  
+
   $(document).keypress(function(ev)
     {
       var key = String.fromCharCode(ev.charCode);
@@ -754,11 +1127,21 @@ ItemCollectionManager.method("setup_keyboard_nav", function()
         return;
       }
 
-      if (self.kb_shortcut_inhibitions != 0)
+      // don't take the kbd shortcut if an input is focused
+      if ($("input:focus").length || $("textarea:focus").length)
         return true;
 
       if (key == "s" || key =="/")
+      {
         $("#search").get(0).focus();
+        handled = true;
+      }
+
+      if (key == "u")
+      {
+        self.reload();
+        handled = true;
+      }
 
       if (key == "c")
       {
@@ -783,11 +1166,20 @@ ItemCollectionManager.method("setup_keyboard_nav", function()
           handled = true;
         }
         else if (key == "e")
+        {
           self.cursor_at.find(".editcontrols .editcontrols_top .editbutton").click();
+          handled = true;
+        }
         else if (key == "h")
+        {
           self.cursor_at.find(".editcontrols .editcontrols_top .item_history_button").click();
+          handled = true;
+        }
         else if (key == "D")
+        {
           self.cursor_at.find(".editcontrols .editcontrols_top .deletebutton").click();
+          handled = true;
+        }
         else if (key == "n")
         {
           var new_item_div = $("#item_null");
@@ -796,6 +1188,7 @@ ItemCollectionManager.method("setup_keyboard_nav", function()
             self.set_cursor_to(new_item_div);
             self.cursor_at.find(".editcontrols .editcontrols_top .editbutton").click();
           }
+          handled = true;
         }
       }
 
@@ -816,6 +1209,10 @@ ItemCollectionManager.method("setup_toolbar", function()
 
       window.open(printurl);
     });
+  $("#btn_calendar").click(function()
+    {
+      window.open("calendar");
+    });
   $("#btn_export").click(function()
     {
       var exporturl = "items/export?query="+escape($("#search").val());
@@ -825,19 +1222,19 @@ ItemCollectionManager.method("setup_toolbar", function()
       window.open(exporturl);
     });
   $("#btn_expand").click(function()
-    { 
+    {
       $(".item").removeClass("collapsed");
     });
   $("#btn_collapse").click(function()
-    { 
+    {
       $(".item").addClass("collapsed");
     });
   $("#btn_quit").click(function()
-    { 
+    {
       location.href = "app/quit";
     });
   $("#btn_copy").click(function()
-    { 
+    {
       var linkurl = $("#linkurl");
       if (linkurl.hasClass("shown"))
       {
@@ -851,7 +1248,6 @@ ItemCollectionManager.method("setup_toolbar", function()
         linkurl_edit.focus();
       }
     });
-  this.install_focus_handlers($("#linkurl_edit"));
 });
 
 
@@ -882,22 +1278,6 @@ ItemCollectionManager.method("set_cursor_to", function(div, is_up, noscroll)
 
 
 
-ItemCollectionManager.method("install_focus_handlers", function(input_el)
-{
-  var self = this;
-  input_el.focus(function(ev) 
-    { 
-      ++self.kb_shortcut_inhibitions; 
-    });
-  input_el.blur(function(ev) 
-    { 
-      --self.kb_shortcut_inhibitions; 
-    });
-});
-
-
-
-
 ItemCollectionManager.method("note_list_emptied", function(item)
 {
   this.cursor_at = null;
@@ -1000,6 +1380,13 @@ ItemCollectionManager.method("update", function(force)
 
 
 
+ItemCollectionManager.method("reload", function()
+{
+  this.update(true);
+})
+
+
+
 
 ItemCollectionManager.method("fill", function(query, timestamp, force)
 {
@@ -1010,6 +1397,13 @@ ItemCollectionManager.method("fill", function(query, timestamp, force)
   }
 
   var self = this;
+
+  while (self.item_timeouts.length)
+  {
+    timeout = self.item_timeouts.pop();
+    window.clearTimeout(timeout);
+  }
+
   if (!force && query == self.last_query && timestamp == self.last_timestamp)
     return;
 
@@ -1047,6 +1441,10 @@ ItemCollectionManager.method("fill", function(query, timestamp, force)
 
       fill_subtag_cloud(json);
       self.query_tags = json.query_tags;
+      self.last_query_data = data;
+      self.result_hash = json.result_hash;
+
+      $("#out_of_date_notifier").addClass("ood_hidden");
     },
     error: function(req, stat, err)
     {
@@ -1069,7 +1467,68 @@ ItemCollectionManager.method("fill", function(query, timestamp, force)
 
 
 
-// tag cloud -------------------------------------------------------------------
+ItemCollectionManager.method("check_if_results_are_current", function()
+{
+  var self = this;
+
+  if (self.last_query_data && self.result_hash
+    && self.up_to_date_check_inhibitions == 0)
+  {
+    $.ajax({
+      data: self.last_query_data,
+      url: 'items/get_result_hash',
+      dataType:"json",
+      success: function(json, status)
+      {
+        if (self.result_hash != json
+          && self.up_to_date_check_inhibitions == 0)
+        {
+          $("#out_of_date_notifier").removeClass("ood_hidden");
+        }
+      }
+    });
+  }
+
+  window.setTimeout(function() { self.check_if_results_are_current() },
+      UP_TO_DATE_TIMEOUT);
+});
+
+
+
+
+ItemCollectionManager.method("update_result_hash_and_uninhibit_check", function()
+{
+  var self = this;
+
+  if (!$("#out_of_date_notifier").hasClass("ood_hidden"))
+  {
+    // we are out of date, do nothing
+    return
+  }
+
+  if (self.last_query_data && self.result_hash)
+  {
+    $.ajax({
+      data: self.last_query_data,
+      url: 'items/get_result_hash',
+      dataType:"json",
+      success: function(json, status)
+      {
+        self.result_hash = json;
+        --self.up_to_date_check_inhibitions;
+      },
+      error: function()
+      {
+        --self.up_to_date_check_inhibitions;
+      }
+    });
+  }
+});
+
+// }}}
+
+// {{{ tag cloud
+
 function is_valid_tag(tag)
 {
   return tag.match(/^[.a-zA-Z0-9]+$/) != null;
@@ -1078,7 +1537,7 @@ function is_valid_tag(tag)
 
 
 
-function format_tag_links(tags, joiner) 
+function format_tag_links(tags, joiner)
 {
   if (joiner == undefined)
     joiner = ", ";
@@ -1086,7 +1545,7 @@ function format_tag_links(tags, joiner)
   var result = '';
   for (var i = 0; i < tags.length; ++i)
   {
-    result += '<a class="taglink">'+tags[i]+'</a>';
+    result += '<a class="taglink tag-'+tags[i]+' ui-corner-all">'+tags[i]+'</a>';
     if (i < tags.length-1)
       result += joiner;
   }
@@ -1130,7 +1589,7 @@ function make_tag_cloud(data, show_hidden, exclude)
     var sizefraction = 1-Math.pow(1-usefraction, 0.8);
     var fontsize = Math.round(7.+usefraction*15);
 
-    html += ('<a class="taglink" style="font-size:[fs]pt">[tag]</a> '
+    html += ('<a class="taglink tag-[tag]" style="font-size:[fs]pt">[tag]</a> '
       .allreplace("[fs]", fontsize)
       .allreplace("[tag]", tag));
   }
@@ -1153,7 +1612,7 @@ function update_tag_clouds()
 function update_main_tag_cloud()
 {
   $.getJSON("tags/get_for_query", function (json)
-    {  
+    {
       $("#tagcloud").html(make_tag_cloud(
           json.tags, $("#chk_tagcloud_show_hidden").get(0).checked));
       add_tag_behavior($("#tagcloud a"));
@@ -1166,8 +1625,8 @@ function update_main_tag_cloud()
 function fill_subtag_cloud(data)
 {
   $("#subtagcloud").html(
-    make_tag_cloud(data.tags, 
-      $("#chk_subtagcloud_show_hidden").get(0).checked, 
+    make_tag_cloud(data.tags,
+      $("#chk_subtagcloud_show_hidden").get(0).checked,
       data.query_tags));
   add_tag_behavior($("#subtagcloud a"));
 
@@ -1258,7 +1717,7 @@ function add_tag_behavior(jq_result)
     { set_search($(this).html()); });
   jq_result.contextMenu("tag_context_menu", {
       bindings: {
-        'tag-menu-rename': function(tag_html) 
+        'tag-menu-rename': function(tag_html)
         {
           var old_name = $(tag_html).html();
           var new_name = prompt(
@@ -1267,26 +1726,23 @@ function add_tag_behavior(jq_result)
             old_name);
           if (new_name != undefined && new_name != old_name)
           {
-            if (!is_valid_tag(new_name))
-              alert("Cannot rename--not a valid tag.");
-            else
-            {
-              $.ajax({
-                type: 'POST',
-                dataType: 'text',
-                url: 'tags/rename',
-                data: {json: JSON.stringify({
-                  old_name: old_name,
-                  new_name: new_name
-                })},
-                error: function(req, stat, err) { report_error("Rename failed."); },
-                success: function(data, msg) { 
-                  set_message("Rename successful."); 
-                  update_tag_clouds();
-                  document.collection_manager.update(true);
-                }
-              });
-            }
+            $.ajax({
+              type: 'POST',
+              dataType: 'text',
+              url: 'tags/rename',
+              data: {json: JSON.stringify({
+                old_name: old_name,
+                new_name: new_name
+              })},
+              error: function(req, stat, err) {
+                alert("Rename failed.\n"+req.responseText);
+              },
+              success: function(data, msg) {
+                set_message("Rename successful.");
+                update_tag_clouds();
+                document.collection_manager.update(true);
+              }
+            });
           }
         },
         'tag-menu-restrict': function(tag_html) {
@@ -1302,7 +1758,6 @@ function add_tag_behavior(jq_result)
 
 
 
-// functions  ------------------------------------------------------------------
 function update_tag_cloud_height()
 {
   var cloud_bottom_offset;
@@ -1319,7 +1774,7 @@ function update_tag_cloud_height()
       $(window).height()
       - $("#subtagcloud").offset().top
       - bottom_size
-      - 20
+      - 50
       );
 
   $(".tagcloud").css("max-height", tc_height+"px");
@@ -1327,6 +1782,10 @@ function update_tag_cloud_height()
 
 
 
+// }}}
+
+
+
 
 $(document).ready(function()
 {
@@ -1335,11 +1794,11 @@ $(document).ready(function()
 
   tag_tabs = $("#navtabs").tabs();
 
-  $("#items").sortable({ 
+  $("#items").sortable({
     handle: ".item-drag-handle",
-    start: function(event, ui) 
+    start: function(event, ui)
       { $("#item_null").appendTo("#hiddenitems"); },
-    stop: function(event, ui) 
+    stop: function(event, ui)
       { $("#item_null").appendTo("#items"); },
     update: function(event, ui)
       {
@@ -1357,7 +1816,7 @@ $(document).ready(function()
               new_order: item_ids.join(","),
               current_search: $("#search").val()
             })},
-          error: function(req, stat, err) 
+          error: function(req, stat, err)
           { report_error("Reordering failed on server."); }
         });
       }
@@ -1370,8 +1829,8 @@ $(document).ready(function()
   dhtmlHistory.initialize();
   dhtmlHistory.addListener(
     function (new_loc, data)
-    { 
-      collection_manager.handle_history_event(new_loc, data); 
+    {
+      collection_manager.handle_history_event(new_loc, data);
     });
   if (dhtmlHistory.isFirstLoad())
     collection_manager.handle_history_event(dhtmlHistory.getCurrentLocation(), null);
@@ -1379,7 +1838,7 @@ $(document).ready(function()
   $(window).resize(update_tag_cloud_height);
   update_tag_cloud_height();
 
-  window.onbeforeunload = function(evt) { 
+  window.onbeforeunload = function(evt) {
     var e = e || window.event;
 
     if (nav_warning_inhibitor == 0 && edit_count)
@@ -1393,12 +1852,14 @@ $(document).ready(function()
 
 
 window.dhtmlHistory.create({
-  toJSON: function(o) 
+  toJSON: function(o)
   {
     return JSON.stringify(o);
-  }, 
-  fromJSON: function(s) 
+  },
+  fromJSON: function(s)
   {
     return JSON.parse(s);
   }
 });
+
+// vim: foldmethod=marker
